@@ -22,13 +22,13 @@
  */
 
 #include <vmm_error.h>
-#include <vmm_list.h>
 #include <vmm_heap.h>
-#include <vmm_string.h>
 #include <vmm_devtree.h>
 #include <vmm_devemu.h>
 #include <vmm_host_aspace.h>
 #include <vmm_guest_aspace.h>
+#include <vmm_stdio.h>
+#include <libs/stringlib.h>
 
 struct vmm_region *vmm_guest_find_region(struct vmm_guest *guest,
 					 physical_addr_t gphys_addr,
@@ -76,13 +76,13 @@ struct vmm_region *vmm_guest_find_region(struct vmm_guest *guest,
 	return reg;
 }
 
-u32 vmm_guest_physical_read(struct vmm_guest * guest, 
+u32 vmm_guest_physical_read(struct vmm_guest *guest, 
 			    physical_addr_t gphys_addr, 
-			    void * dst, u32 len)
+			    void *dst, u32 len)
 {
 	u32 bytes_read = 0, to_read;
 	physical_addr_t hphys_addr;
-	struct vmm_region * reg = NULL;
+	struct vmm_region *reg = NULL;
 
 	if (!guest || !dst || !len) {
 		return 0;
@@ -116,13 +116,13 @@ u32 vmm_guest_physical_read(struct vmm_guest * guest,
 	return bytes_read;
 }
 
-u32 vmm_guest_physical_write(struct vmm_guest * guest, 
+u32 vmm_guest_physical_write(struct vmm_guest *guest, 
 			     physical_addr_t gphys_addr, 
-			     void * src, u32 len)
+			     void *src, u32 len)
 {
 	u32 bytes_written = 0, to_write;
 	physical_addr_t hphys_addr;
-	struct vmm_region * reg = NULL;
+	struct vmm_region *reg = NULL;
 
 	if (!guest || !src || !len) {
 		return 0;
@@ -156,15 +156,15 @@ u32 vmm_guest_physical_write(struct vmm_guest * guest,
 	return bytes_written;
 }
 
-int vmm_guest_physical_map(struct vmm_guest * guest,
+int vmm_guest_physical_map(struct vmm_guest *guest,
 			   physical_addr_t gphys_addr,
 			   physical_size_t gphys_size,
-			   physical_addr_t * hphys_addr,
-			   physical_size_t * hphys_size,
-			   u32 * reg_flags)
+			   physical_addr_t *hphys_addr,
+			   physical_size_t *hphys_size,
+			   u32 *reg_flags)
 {
 	/* FIXME: Need to implement dynamic RAM allocation for RAM region */
-	struct vmm_region * reg = NULL;
+	struct vmm_region *reg = NULL;
 
 	if (!guest || !hphys_addr) {
 		return VMM_EFAIL;
@@ -198,7 +198,7 @@ int vmm_guest_physical_map(struct vmm_guest * guest,
 	return VMM_OK;
 }
 
-int vmm_guest_physical_unmap(struct vmm_guest * guest,
+int vmm_guest_physical_unmap(struct vmm_guest *guest,
 			     physical_addr_t gphys_addr,
 			     physical_size_t gphys_size)
 {
@@ -206,67 +206,6 @@ int vmm_guest_physical_unmap(struct vmm_guest * guest,
 	return VMM_OK;
 }
 
-bool is_address_node_valid(struct vmm_devtree_node * anode)
-{
-	const char *attrval;
-	bool is_real = FALSE;
-	bool is_alias = FALSE;
-
-	attrval = vmm_devtree_attrval(anode, VMM_DEVTREE_MANIFEST_TYPE_ATTR_NAME);
-	if (!attrval) {
-		return FALSE;
-	}
-	if (vmm_strcmp(attrval, VMM_DEVTREE_MANIFEST_TYPE_VAL_REAL) != 0 &&
-	    vmm_strcmp(attrval, VMM_DEVTREE_MANIFEST_TYPE_VAL_VIRTUAL) != 0 && 
-	    vmm_strcmp(attrval, VMM_DEVTREE_MANIFEST_TYPE_VAL_ALIAS) != 0) {
-		return FALSE;
-	}
-	if (vmm_strcmp(attrval, VMM_DEVTREE_MANIFEST_TYPE_VAL_REAL) == 0) {
-		is_real = TRUE;
-	}
-	if (vmm_strcmp(attrval, VMM_DEVTREE_MANIFEST_TYPE_VAL_ALIAS) == 0) {
-		is_alias = TRUE;
-	}
-
-	attrval = vmm_devtree_attrval(anode,
-				      VMM_DEVTREE_ADDRESS_TYPE_ATTR_NAME);
-	if (!attrval) {
-		return FALSE;
-	}
-	if (vmm_strcmp(attrval, VMM_DEVTREE_ADDRESS_TYPE_VAL_IO) != 0 &&
-	    vmm_strcmp(attrval, VMM_DEVTREE_ADDRESS_TYPE_VAL_MEMORY) != 0) {
-		return FALSE;
-	}
-
-	attrval = vmm_devtree_attrval(anode, VMM_DEVTREE_GUEST_PHYS_ATTR_NAME);
-	if (!attrval) {
-		return FALSE;
-	}
-
-	if (is_real) {
-		attrval = vmm_devtree_attrval(anode, 
-					VMM_DEVTREE_HOST_PHYS_ATTR_NAME);
-		if (!attrval) {
-			return FALSE;
-		}
-	}
-
-	if (is_alias) {
-		attrval = vmm_devtree_attrval(anode, 
-					VMM_DEVTREE_ALIAS_PHYS_ATTR_NAME);
-		if (!attrval) {
-			return FALSE;
-		}
-	}
-
-	attrval = vmm_devtree_attrval(anode, VMM_DEVTREE_PHYS_SIZE_ATTR_NAME);
-	if (!attrval) {
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
 int vmm_guest_aspace_reset(struct vmm_guest *guest)
 {
 	struct dlist *l;
@@ -289,13 +228,101 @@ int vmm_guest_aspace_reset(struct vmm_guest *guest)
 	return vmm_devemu_reset_context(guest);
 }
 
+bool is_region_node_valid(struct vmm_devtree_node *rnode)
+{
+	const char *attrval;
+	bool is_real = FALSE;
+	bool is_alias = FALSE;
+
+	attrval = vmm_devtree_attrval(rnode, 
+				      VMM_DEVTREE_MANIFEST_TYPE_ATTR_NAME);
+	if (!attrval) {
+		return FALSE;
+	}
+	if (strcmp(attrval, VMM_DEVTREE_MANIFEST_TYPE_VAL_REAL) != 0 &&
+	    strcmp(attrval, VMM_DEVTREE_MANIFEST_TYPE_VAL_VIRTUAL) != 0 && 
+	    strcmp(attrval, VMM_DEVTREE_MANIFEST_TYPE_VAL_ALIAS) != 0) {
+		return FALSE;
+	}
+	if (strcmp(attrval, VMM_DEVTREE_MANIFEST_TYPE_VAL_REAL) == 0) {
+		is_real = TRUE;
+	}
+	if (strcmp(attrval, VMM_DEVTREE_MANIFEST_TYPE_VAL_ALIAS) == 0) {
+		is_alias = TRUE;
+	}
+
+	attrval = vmm_devtree_attrval(rnode,
+				      VMM_DEVTREE_ADDRESS_TYPE_ATTR_NAME);
+	if (!attrval) {
+		return FALSE;
+	}
+	if (strcmp(attrval, VMM_DEVTREE_ADDRESS_TYPE_VAL_IO) != 0 &&
+	    strcmp(attrval, VMM_DEVTREE_ADDRESS_TYPE_VAL_MEMORY) != 0) {
+		return FALSE;
+	}
+
+	attrval = vmm_devtree_attrval(rnode, VMM_DEVTREE_GUEST_PHYS_ATTR_NAME);
+	if (!attrval) {
+		return FALSE;
+	}
+
+	if (is_real) {
+		attrval = vmm_devtree_attrval(rnode, 
+					      VMM_DEVTREE_HOST_PHYS_ATTR_NAME);
+		if (!attrval) {
+			return FALSE;
+		}
+	}
+
+	if (is_alias) {
+		attrval = vmm_devtree_attrval(rnode, 
+					     VMM_DEVTREE_ALIAS_PHYS_ATTR_NAME);
+		if (!attrval) {
+			return FALSE;
+		}
+	}
+
+	attrval = vmm_devtree_attrval(rnode, VMM_DEVTREE_PHYS_SIZE_ATTR_NAME);
+	if (!attrval) {
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+static bool is_region_overlapping(struct vmm_guest *guest,
+				  struct vmm_region *reg)
+{
+	struct dlist *l;
+	struct vmm_region *treg = NULL;
+	physical_addr_t reg_start, reg_end;
+	physical_addr_t treg_start, treg_end;
+
+	reg_start = reg->gphys_addr;
+	reg_end = reg->gphys_addr + reg->phys_size;
+
+	list_for_each(l, &guest->aspace.reg_list) {
+		treg = list_entry(l, struct vmm_region, head);
+		treg_start = treg->gphys_addr;
+		treg_end = treg->gphys_addr + treg->phys_size;
+		if ((treg_start <= reg_start) && (reg_start < treg_end)) {
+			return TRUE;
+		}
+		if ((treg_start <= reg_end) && (reg_end < treg_end)) {
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
 int vmm_guest_aspace_init(struct vmm_guest *guest)
 {
 	int rc;
 	const char *attrval;
 	struct dlist *l;
 	struct vmm_devtree_node *gnode = guest->node;
-	struct vmm_devtree_node *anode = NULL;
+	struct vmm_devtree_node *rnode = NULL;
 	struct vmm_region *reg = NULL;
 
 	/* Sanity Check */
@@ -304,12 +331,13 @@ int vmm_guest_aspace_init(struct vmm_guest *guest)
 	}
 
 	/* Reset the address space for guest */
-	vmm_memset(&guest->aspace, 0, sizeof(struct vmm_guest_aspace));
+	memset(&guest->aspace, 0, sizeof(struct vmm_guest_aspace));
 
 	/* Get address space node under guest node */
 	guest->aspace.node = vmm_devtree_getchild(gnode, 
 					VMM_DEVTREE_ADDRSPACE_NODE_NAME);
 	if (!guest->aspace.node) {
+		vmm_printf("%s: %s/aspace node not found\n", __func__, gnode->name);
 		return VMM_EFAIL;
 	}
 
@@ -322,50 +350,53 @@ int vmm_guest_aspace_init(struct vmm_guest *guest)
 	/* Initialize devemu_priv pointer of aspace */
 	guest->aspace.devemu_priv = NULL;
 
+	/* Initialize device emulation context */
+	if ((rc = vmm_devemu_init_context(guest))) {
+		return rc;
+	}
+
 	/* Populate valid regions */
 	list_for_each(l, &(guest->aspace.node->child_list)) {
-		anode = list_entry(l, struct vmm_devtree_node, head);
-		if (!anode) {
+		rnode = list_entry(l, struct vmm_devtree_node, head);
+		if (!rnode) {
 			continue;
 		}
 
-		if (!is_address_node_valid(anode)) {
+		if (!is_region_node_valid(rnode)) {
 			continue;
 		}
 
-		reg = vmm_malloc(sizeof(struct vmm_region));
-
-		vmm_memset(reg, 0, sizeof(struct vmm_region));
+		reg = vmm_zalloc(sizeof(struct vmm_region));
 
 		INIT_LIST_HEAD(&reg->head);
 
-		reg->node = anode;
+		reg->node = rnode;
 		reg->aspace = &guest->aspace;
 		reg->flags = 0x0;
 
-		attrval = vmm_devtree_attrval(anode,
+		attrval = vmm_devtree_attrval(rnode,
 					      VMM_DEVTREE_MANIFEST_TYPE_ATTR_NAME);
-		if (vmm_strcmp(attrval, VMM_DEVTREE_MANIFEST_TYPE_VAL_REAL) == 0) {
+		if (strcmp(attrval, VMM_DEVTREE_MANIFEST_TYPE_VAL_REAL) == 0) {
 			reg->flags |= VMM_REGION_REAL;
-		} else if (vmm_strcmp(attrval, VMM_DEVTREE_MANIFEST_TYPE_VAL_ALIAS) == 0) {
+		} else if (strcmp(attrval, VMM_DEVTREE_MANIFEST_TYPE_VAL_ALIAS) == 0) {
 			reg->flags |= VMM_REGION_ALIAS;
 		} else {
 			reg->flags |= VMM_REGION_VIRTUAL;
 		}
 
-		attrval = vmm_devtree_attrval(anode,
+		attrval = vmm_devtree_attrval(rnode,
 					      VMM_DEVTREE_ADDRESS_TYPE_ATTR_NAME);
-		if (vmm_strcmp(attrval, VMM_DEVTREE_ADDRESS_TYPE_VAL_IO) == 0) {
+		if (strcmp(attrval, VMM_DEVTREE_ADDRESS_TYPE_VAL_IO) == 0) {
 			reg->flags |= VMM_REGION_IO;
 		} else {
 			reg->flags |= VMM_REGION_MEMORY;
 		}
 
-		attrval = vmm_devtree_attrval(anode,
+		attrval = vmm_devtree_attrval(rnode,
 					      VMM_DEVTREE_DEVICE_TYPE_ATTR_NAME);
-		if (vmm_strcmp(attrval, VMM_DEVTREE_DEVICE_TYPE_VAL_RAM) == 0) {
+		if (strcmp(attrval, VMM_DEVTREE_DEVICE_TYPE_VAL_RAM) == 0) {
 			reg->flags |= VMM_REGION_ISRAM;
-		} else if (vmm_strcmp(attrval, VMM_DEVTREE_DEVICE_TYPE_VAL_ROM) == 0) {
+		} else if (strcmp(attrval, VMM_DEVTREE_DEVICE_TYPE_VAL_ROM) == 0) {
 			reg->flags |= VMM_REGION_READONLY;
 			reg->flags |= VMM_REGION_ISROM;
 		} else {
@@ -379,50 +410,57 @@ int vmm_guest_aspace_init(struct vmm_guest *guest)
 			reg->flags |= VMM_REGION_BUFFERABLE;
 		}
 
-		attrval = vmm_devtree_attrval(anode,
+		attrval = vmm_devtree_attrval(rnode,
 					      VMM_DEVTREE_GUEST_PHYS_ATTR_NAME);
 		reg->gphys_addr = *((physical_addr_t *) attrval);
 
 		if (reg->flags & VMM_REGION_REAL) {
-			attrval = vmm_devtree_attrval(anode,
+			attrval = vmm_devtree_attrval(rnode,
 					      VMM_DEVTREE_HOST_PHYS_ATTR_NAME);
 			reg->hphys_addr = *((physical_addr_t *) attrval);
 		} else if (reg->flags & VMM_REGION_ALIAS){
-			attrval = vmm_devtree_attrval(anode,
+			attrval = vmm_devtree_attrval(rnode,
 					      VMM_DEVTREE_ALIAS_PHYS_ATTR_NAME);
 			reg->hphys_addr = *((physical_addr_t *) attrval);
 		} else {
 			reg->hphys_addr = reg->gphys_addr;
 		}
 
-		attrval = vmm_devtree_attrval(anode,
+		attrval = vmm_devtree_attrval(rnode,
 					      VMM_DEVTREE_PHYS_SIZE_ATTR_NAME);
 		reg->phys_size = *((physical_size_t *) attrval);
 
 		reg->devemu_priv = NULL;
 
+		if (is_region_overlapping(guest, reg)) {
+			vmm_free(reg);
+			vmm_printf("%s: Region for %s/%s overlapping with a previous node\n", 
+					__func__, gnode->name, rnode->name);
+			return VMM_EINVALID;
+		}
+
 		if (!(reg->flags & (VMM_REGION_ALIAS | VMM_REGION_VIRTUAL)) && 
 		    (reg->flags & (VMM_REGION_ISRAM | VMM_REGION_ISROM))) {
 			rc = vmm_host_ram_reserve(reg->hphys_addr, 
 						  reg->phys_size);
 			if (rc) {
+				vmm_printf("%s: Failed to reserve physical region for %s/%s\n", 
+						__func__, gnode->name, rnode->name);
+				vmm_free(reg);
 				return rc;
 			}
 		}
 
-		list_add_tail(&guest->aspace.reg_list, &reg->head);
+		list_add_tail(&reg->head, &guest->aspace.reg_list);
 	}
 
-	/* Initialize device emulation context */
-	if ((rc = vmm_devemu_init_context(guest))) {
-		return rc;
-	}
-
-	/* Probe device emulation for virutal regions */
+	/* Probe device emulation for virtual regions */
 	list_for_each(l, &guest->aspace.reg_list) {
 		reg = list_entry(l, struct vmm_region, head);
 		if (reg->flags & VMM_REGION_VIRTUAL) {
-			vmm_devemu_probe_region(guest, reg);
+			if ((rc = vmm_devemu_probe_region(guest, reg))) {
+				return rc;
+			}
 		}
 	}
 
