@@ -22,21 +22,23 @@
  */
 
 #include <vmm_error.h>
-#include <vmm_string.h>
 #include <vmm_heap.h>
+#include <vmm_delay.h>
+#include <vmm_compiler.h>
 #include <vmm_scheduler.h>
 #include <vmm_workqueue.h>
+#include <libs/stringlib.h>
 
 struct vmm_workqueue_ctrl {
 	vmm_spinlock_t lock;
 	struct dlist wq_list;
 	u32 wq_count;
-	struct vmm_workqueue * syswq;
+	struct vmm_workqueue *syswq;
 };
 
 static struct vmm_workqueue_ctrl wqctrl;
 
-bool vmm_workqueue_work_isnew(struct vmm_work * work)
+bool vmm_workqueue_work_isnew(struct vmm_work *work)
 {
 	bool ret = FALSE;
 	irq_flags_t flags;
@@ -45,7 +47,7 @@ bool vmm_workqueue_work_isnew(struct vmm_work * work)
 		return FALSE;
 	}
 
-	flags = vmm_spin_lock_irqsave(&work->lock);
+	vmm_spin_lock_irqsave(&work->lock, flags);
 
 	ret = (work->flags & VMM_WORK_STATE_CREATED) ? TRUE : FALSE;
 
@@ -54,7 +56,7 @@ bool vmm_workqueue_work_isnew(struct vmm_work * work)
 	return ret;
 }
 
-bool vmm_workqueue_work_inprogress(struct vmm_work * work)
+bool vmm_workqueue_work_inprogress(struct vmm_work *work)
 {
 	bool ret = FALSE;
 	irq_flags_t flags;
@@ -63,7 +65,7 @@ bool vmm_workqueue_work_inprogress(struct vmm_work * work)
 		return FALSE;
 	}
 
-	flags = vmm_spin_lock_irqsave(&work->lock);
+	vmm_spin_lock_irqsave(&work->lock, flags);
 
 	ret = (work->flags & VMM_WORK_STATE_INPROGRESS) ? TRUE : FALSE;
 
@@ -72,7 +74,7 @@ bool vmm_workqueue_work_inprogress(struct vmm_work * work)
 	return ret;
 }
 
-bool vmm_workqueue_work_completed(struct vmm_work * work)
+bool vmm_workqueue_work_completed(struct vmm_work *work)
 {
 	bool ret = FALSE;
 	irq_flags_t flags;
@@ -81,7 +83,7 @@ bool vmm_workqueue_work_completed(struct vmm_work * work)
 		return FALSE;
 	}
 
-	flags = vmm_spin_lock_irqsave(&work->lock);
+	vmm_spin_lock_irqsave(&work->lock, flags);
 
 	ret = (work->flags & VMM_WORK_STATE_STOPPED) ? TRUE : FALSE;
 
@@ -90,7 +92,7 @@ bool vmm_workqueue_work_completed(struct vmm_work * work)
 	return ret;
 }
 
-int vmm_workqueue_stop_work(struct vmm_work * work)
+int vmm_workqueue_stop_work(struct vmm_work *work)
 {
 	irq_flags_t flags, flags1;
 
@@ -98,16 +100,22 @@ int vmm_workqueue_stop_work(struct vmm_work * work)
 		return VMM_EFAIL;
 	}
 
-	flags = vmm_spin_lock_irqsave(&work->lock);
+stop_retry:
+	vmm_spin_lock_irqsave(&work->lock, flags);
 
-	if (work->flags & 
-		(VMM_WORK_STATE_INPROGRESS | VMM_WORK_STATE_STOPPED)) {
+	if (work->flags & VMM_WORK_STATE_INPROGRESS) {
+		vmm_spin_unlock_irqrestore(&work->lock, flags);
+		vmm_udelay(VMM_THREAD_DEF_TIME_SLICE / 1000);
+		goto stop_retry;
+	}
+
+	if (work->flags & VMM_WORK_STATE_STOPPED) {
 		vmm_spin_unlock_irqrestore(&work->lock, flags);
 		return VMM_OK;
 	}
 
 	if (work->wq && (work->flags & VMM_WORK_STATE_SCHEDULED)) {
-		flags1 = vmm_spin_lock_irqsave(&(work->wq)->lock);
+		vmm_spin_lock_irqsave(&(work->wq)->lock, flags1);
 		list_del(&work->head);
 		vmm_spin_unlock_irqrestore(&(work->wq)->lock, flags1);
 	}
@@ -120,7 +128,23 @@ int vmm_workqueue_stop_work(struct vmm_work * work)
 	return VMM_OK;
 }
 
-struct vmm_thread *vmm_workqueue_get_thread(struct vmm_workqueue * wq)
+int vmm_workqueue_stop_delayed_work(struct vmm_delayed_work *work)
+{
+	int rc;
+
+	if (!work) {
+		return VMM_EFAIL;
+	}
+
+	rc = vmm_timer_event_stop(&work->event);
+	if (rc) {
+		return rc;
+	}
+
+	return vmm_workqueue_stop_work(&work->work);
+}
+
+struct vmm_thread *vmm_workqueue_get_thread(struct vmm_workqueue *wq)
 {
 	return (wq) ? wq->thread : NULL;
 }
@@ -139,7 +163,7 @@ struct vmm_workqueue *vmm_workqueue_index2workqueue(int index)
 	ret = NULL;
 	found = FALSE;
 
-	flags = vmm_spin_lock_irqsave(&wqctrl.lock);
+	vmm_spin_lock_irqsave(&wqctrl.lock, flags);
 
 	list_for_each(l, &wqctrl.wq_list) {
 		ret = list_entry(l, struct vmm_workqueue, head);
@@ -164,7 +188,7 @@ u32 vmm_workqueue_count(void)
 	return wqctrl.wq_count;
 }
 
-int vmm_workqueue_flush(struct vmm_workqueue * wq)
+int vmm_workqueue_flush(struct vmm_workqueue *wq)
 {
 	int rc;
 
@@ -184,8 +208,8 @@ int vmm_workqueue_flush(struct vmm_workqueue * wq)
 	return VMM_OK;
 }
 
-int vmm_workqueue_schedule_work(struct vmm_workqueue * wq, 
-				struct vmm_work * work)
+int vmm_workqueue_schedule_work(struct vmm_workqueue *wq, 
+				struct vmm_work *work)
 {
 	irq_flags_t flags, flags1;
 
@@ -194,12 +218,10 @@ int vmm_workqueue_schedule_work(struct vmm_workqueue * wq,
 	}
 
 	if (!wq) {
-		/* For SMP we will have to do load balancing 
-		 * among multiple system workqueues */
 		wq = wqctrl.syswq;
 	}
 
-	flags = vmm_spin_lock_irqsave(&work->lock);
+	vmm_spin_lock_irqsave(&work->lock, flags);
 
 	if ((work->flags != VMM_WORK_STATE_CREATED) &&
 	    (work->flags != VMM_WORK_STATE_STOPPED)) {
@@ -210,8 +232,8 @@ int vmm_workqueue_schedule_work(struct vmm_workqueue * wq,
 	work->flags = VMM_WORK_STATE_SCHEDULED;
 	work->wq = wq;
 
-	flags1 = vmm_spin_lock_irqsave(&wq->lock);
-	list_add_tail(&wq->work_list, &work->head);
+	vmm_spin_lock_irqsave(&wq->lock, flags1);
+	list_add_tail(&work->head, &wq->work_list);
 	vmm_spin_unlock_irqrestore(&wq->lock, flags1);
 
 	vmm_spin_unlock_irqrestore(&work->lock, flags);
@@ -221,20 +243,45 @@ int vmm_workqueue_schedule_work(struct vmm_workqueue * wq,
 	return VMM_OK;
 }
 
+static void delayed_work_timer_event(struct vmm_timer_event *ev)
+{
+	struct vmm_delayed_work *work = ev->priv;
+
+	vmm_workqueue_schedule_work(work->work.wq, &work->work);
+}
+
+int vmm_workqueue_schedule_delayed_work(struct vmm_workqueue *wq, 
+					struct vmm_delayed_work *work,
+					u64 nsecs)
+{
+	if (!wq || !work) {
+		return VMM_EFAIL;
+	}
+
+	if (!nsecs) {
+		return vmm_workqueue_schedule_work(wq, &work->work);
+	}
+
+	work->work.wq = wq;
+	INIT_TIMER_EVENT(&work->event, delayed_work_timer_event, work);
+
+	return vmm_timer_event_start(&work->event, nsecs);
+}
+
 static int workqueue_main(void *data)
 {
 	bool do_work;
 	irq_flags_t flags;
-	struct dlist * l;
-	struct vmm_workqueue * wq = data;
-	struct vmm_work * work = NULL;
+	struct dlist *l;
+	struct vmm_workqueue *wq = data;
+	struct vmm_work *work = NULL;
 
 	if (!wq) {
 		return VMM_EFAIL;
 	}
 
 	while (1) {
-		flags = vmm_spin_lock_irqsave(&wq->lock);
+		vmm_spin_lock_irqsave(&wq->lock, flags);
 		
 		while (!list_empty(&wq->work_list)) {
 			l = list_pop(&wq->work_list);
@@ -242,7 +289,7 @@ static int workqueue_main(void *data)
 
 			work = list_entry(l, struct vmm_work, head);
 			do_work = FALSE;
-			flags = vmm_spin_lock_irqsave(&work->lock);
+			vmm_spin_lock_irqsave(&work->lock, flags);
 			if (work->flags & VMM_WORK_STATE_SCHEDULED) {
 				work->flags = VMM_WORK_STATE_INPROGRESS;
 				do_work = TRUE;
@@ -251,12 +298,12 @@ static int workqueue_main(void *data)
 
 			if (do_work) {
 				work->func(work);
-				flags = vmm_spin_lock_irqsave(&work->lock);
+				vmm_spin_lock_irqsave(&work->lock, flags);
 				work->flags = VMM_WORK_STATE_STOPPED;
 				vmm_spin_unlock_irqrestore(&work->lock, flags);
 			}
 
-			flags = vmm_spin_lock_irqsave(&wq->lock);
+			vmm_spin_lock_irqsave(&wq->lock, flags);
 		}
 
 		vmm_spin_unlock_irqrestore(&wq->lock, flags);
@@ -267,9 +314,9 @@ static int workqueue_main(void *data)
 	return VMM_OK;
 }
 
-struct vmm_workqueue * vmm_workqueue_create(const char *name, u8 priority)
+struct vmm_workqueue *vmm_workqueue_create(const char *name, u8 priority)
 {
-	struct vmm_workqueue * wq;
+	struct vmm_workqueue *wq;
 	irq_flags_t flags;
 
 	if (!name) {
@@ -298,9 +345,9 @@ struct vmm_workqueue * vmm_workqueue_create(const char *name, u8 priority)
 		return NULL;
 	}
 
-	flags = vmm_spin_lock_irqsave(&wqctrl.lock);
+	vmm_spin_lock_irqsave(&wqctrl.lock, flags);
 
-	list_add_tail(&wqctrl.wq_list, &wq->head);
+	list_add_tail(&wq->head, &wqctrl.wq_list);
 	wqctrl.wq_count++;
 
 	vmm_spin_unlock_irqrestore(&wqctrl.lock, flags);
@@ -308,7 +355,7 @@ struct vmm_workqueue * vmm_workqueue_create(const char *name, u8 priority)
 	return wq;
 }
 
-int vmm_workqueue_destroy(struct vmm_workqueue * wq)
+int vmm_workqueue_destroy(struct vmm_workqueue *wq)
 {
 	int rc;
 	irq_flags_t flags;
@@ -325,7 +372,7 @@ int vmm_workqueue_destroy(struct vmm_workqueue * wq)
 		return rc;
 	}
 
-	flags = vmm_spin_lock_irqsave(&wqctrl.lock);
+	vmm_spin_lock_irqsave(&wqctrl.lock, flags);
 
 	list_del(&wq->head);
 	wqctrl.wq_count--;
@@ -340,7 +387,7 @@ int vmm_workqueue_destroy(struct vmm_workqueue * wq)
 int __init vmm_workqueue_init(void)
 {
 	/* Reset control structure */
-	vmm_memset(&wqctrl, 0, sizeof(wqctrl));
+	memset(&wqctrl, 0, sizeof(wqctrl));
 
 	/* Initialize lock in control structure */
 	INIT_SPIN_LOCK(&wqctrl.lock);
@@ -351,9 +398,11 @@ int __init vmm_workqueue_init(void)
 	/* Initialize workqueue count */
 	wqctrl.wq_count = 0;
 
-	/* Create system workqueus (Per Host CPU) */
-	wqctrl.syswq = vmm_workqueue_create("syswq/0", 
-					    VMM_THREAD_DEF_PRIORITY);
+	/* Create one system workqueue with thread priority
+	 * higher than default priority.
+	 */
+	wqctrl.syswq = vmm_workqueue_create("syswq", 
+					    VMM_THREAD_DEF_PRIORITY + 1);
 	if (!wqctrl.syswq) {
 		return VMM_EFAIL;
 	}
