@@ -69,6 +69,7 @@ struct gic_state {
 	vmm_spinlock_t lock;
 
 	/* Configuration */
+	enum gic_type type;
 	u8 id[8];
 	u32 num_cpu;
 	u32 num_irq;
@@ -103,19 +104,19 @@ struct gic_state {
 #define GIC_NUM_IRQ(s) ((s)->num_irq)
 #define GIC_SET_ENABLED(s, irq, cm) (s)->irq_state[irq].enabled |= (cm)
 #define GIC_CLEAR_ENABLED(s, irq, cm) (s)->irq_state[irq].enabled &= ~(cm)
-#define GIC_TEST_ENABLED(s, irq, cm) (((s)->irq_state[irq].enabled & (cm)) != 0)
+#define GIC_TEST_ENABLED(s, irq, cm) ((s)->irq_state[irq].enabled & (cm))
 #define GIC_SET_PENDING(s, irq, cm) (s)->irq_state[irq].pending |= (cm)
 #define GIC_CLEAR_PENDING(s, irq, cm) (s)->irq_state[irq].pending &= ~(cm)
-#define GIC_TEST_PENDING(s, irq, cm) (((s)->irq_state[irq].pending & (cm)) != 0)
+#define GIC_TEST_PENDING(s, irq, cm) ((s)->irq_state[irq].pending & (cm))
 #define GIC_SET_ACTIVE(s, irq, cm) (s)->irq_state[irq].active |= (cm)
 #define GIC_CLEAR_ACTIVE(s, irq, cm) (s)->irq_state[irq].active &= ~(cm)
-#define GIC_TEST_ACTIVE(s, irq, cm) (((s)->irq_state[irq].active & (cm)) != 0)
+#define GIC_TEST_ACTIVE(s, irq, cm) ((s)->irq_state[irq].active & (cm))
 #define GIC_SET_MODEL(s, irq) (s)->irq_state[irq].model = 1
 #define GIC_CLEAR_MODEL(s, irq) (s)->irq_state[irq].model = 0
 #define GIC_TEST_MODEL(s, irq) (s)->irq_state[irq].model
 #define GIC_SET_LEVEL(s, irq, cm) (s)->irq_state[irq].level = (cm)
 #define GIC_CLEAR_LEVEL(s, irq, cm) (s)->irq_state[irq].level &= ~(cm)
-#define GIC_TEST_LEVEL(s, irq, cm) (((s)->irq_state[irq].level & (cm)) != 0)
+#define GIC_TEST_LEVEL(s, irq, cm) ((s)->irq_state[irq].level & (cm))
 #define GIC_SET_TRIGGER(s, irq) (s)->irq_state[irq].trigger = 1
 #define GIC_CLEAR_TRIGGER(s, irq) (s)->irq_state[irq].trigger = 0
 #define GIC_TEST_TRIGGER(s, irq) (s)->irq_state[irq].trigger
@@ -124,11 +125,12 @@ struct gic_state {
 #define GIC_TARGET(s, irq) (s)->irq_target[irq]
 
 /* Update interrupt status after enabled or pending bits have been changed. */
-static void gic_update(struct gic_state *s)
+static void __gic_update(struct gic_state *s)
 {
 	int best_irq, best_prio;
 	int irq, level, cpu, cm;
 	struct vmm_vcpu *vcpu;
+
 	for (cpu = 0; cpu < GIC_NUM_CPU(s); cpu++) {
 		cm = 1 << cpu;
 		s->current_pending[cpu] = 1023;
@@ -154,7 +156,7 @@ static void gic_update(struct gic_state *s)
 			}
 		}
 		level = 0;
-		if (best_prio <= s->priority_mask[cpu]) {
+		if (best_prio < s->priority_mask[cpu]) {
 			s->current_pending[cpu] = best_irq;
 			if (best_prio < s->running_priority[cpu]) {
 				level = 1;
@@ -180,12 +182,23 @@ static void gic_update(struct gic_state *s)
 	}
 }
 
+static void gic_update(struct gic_state *s)
+{
+	irq_flags_t flags;
+
+	vmm_spin_lock_irqsave(&s->lock, flags);
+
+	__gic_update(s);
+
+	vmm_spin_unlock_irqrestore(&s->lock, flags);
+}
+
 /* Process IRQ asserted in device emulation framework */
 static int gic_irq_handle(struct vmm_emupic *epic, u32 irq, int cpu, int level)
 {
-	struct gic_state * s = (struct gic_state *)epic->priv;
 	irq_flags_t flags;
 	int cm, target;
+	struct gic_state *s = (struct gic_state *)epic->priv;
 
 	/* Ensure irq is in range (base_irq, base_irq + num_irq) */
 	if ((s->num_base_irq + s->num_irq) <= irq) {
@@ -201,11 +214,13 @@ static int gic_irq_handle(struct vmm_emupic *epic, u32 irq, int cpu, int level)
 		target = GIC_TARGET(s, irq);
 	}	
 
-	if (level == GIC_TEST_LEVEL(s, irq, cm))
-		return VMM_EMUPIC_IRQ_HANDLED;
-
 	vmm_spin_lock_irqsave(&s->lock, flags);
 
+	if (level == GIC_TEST_LEVEL(s, irq, cm)) {
+		vmm_spin_unlock_irqrestore(&s->lock, flags);
+		return VMM_EMUPIC_IRQ_HANDLED;
+	}
+
 	if (level) {
 		GIC_SET_LEVEL(s, irq, cm);
 		if (GIC_TEST_TRIGGER(s, irq) || GIC_TEST_ENABLED(s, irq, cm)) {
@@ -215,14 +230,14 @@ static int gic_irq_handle(struct vmm_emupic *epic, u32 irq, int cpu, int level)
 		GIC_CLEAR_LEVEL(s, irq, cm);
 	}
 
-	gic_update(s);
+	__gic_update(s);
 
 	vmm_spin_unlock_irqrestore(&s->lock, flags);
 
 	return VMM_EMUPIC_IRQ_HANDLED;
 }
 
-static void gic_set_running_irq(struct gic_state *s, int cpu, int irq)
+static void __gic_set_running_irq(struct gic_state *s, int cpu, int irq)
 {
 	s->running_irq[cpu] = irq;
 	if (irq == 1023) {
@@ -230,13 +245,14 @@ static void gic_set_running_irq(struct gic_state *s, int cpu, int irq)
 	} else {
 		s->running_priority[cpu] = GIC_GET_PRIORITY(s, irq, cpu);
 	}
-	gic_update(s);
+	__gic_update(s);
 }
 
 static u32 gic_acknowledge_irq(struct gic_state *s, int cpu)
 {
 	int new_irq;
 	int cm = 1 << cpu;
+	irq_flags_t flags;
 
 	new_irq = s->current_pending[cpu];
 	if ((new_irq == 1023) ||
@@ -245,11 +261,17 @@ static u32 gic_acknowledge_irq(struct gic_state *s, int cpu)
 	}
 
 	s->last_active[new_irq][cpu] = s->running_irq[cpu];
+
+	vmm_spin_lock_irqsave(&s->lock, flags);
+
 	/* Clear pending flags for both level and edge triggered interrupts.
 	 * Level triggered IRQs will be reasserted once they become inactive. */
 	GIC_CLEAR_PENDING(s, new_irq, 
 			  GIC_TEST_MODEL(s, new_irq) ? GIC_ALL_CPU_MASK(s) : cm);
-	gic_set_running_irq(s, cpu, new_irq);
+
+	__gic_set_running_irq(s, cpu, new_irq);
+
+	vmm_spin_unlock_irqrestore(&s->lock, flags);
 
 	return new_irq;
 }
@@ -258,10 +280,13 @@ static void gic_complete_irq(struct gic_state * s, int cpu, int irq)
 {
 	int update = 0;
 	int cm = 1 << cpu;
+	irq_flags_t flags;
 
 	if (s->running_irq[cpu] == 1023)
 		return; /* No active IRQ.  */
 
+	vmm_spin_lock_irqsave(&s->lock, flags);
+
 	if (irq != 1023) {
 		/* Mark level triggered interrupts as pending if 
 		 * they are still raised. */
@@ -285,19 +310,21 @@ static void gic_complete_irq(struct gic_state * s, int cpu, int irq)
 			tmp = s->last_active[tmp][cpu];
 		}
 		if (update) {
-			gic_update(s);
+			__gic_update(s);
 		}
 	} else {
 		/* Complete the current running IRQ.  */
-		gic_set_running_irq(s, cpu, 
+		__gic_set_running_irq(s, cpu, 
 				s->last_active[s->running_irq[cpu]][cpu]);
 	}
 
+	vmm_spin_unlock_irqrestore(&s->lock, flags);
+
 	/* Signal completion of host-to-guest mapped irq */
 	vmm_devemu_complete_h2g_irq(s->guest, irq);
 }
 
-static int gic_dist_readb(struct gic_state * s, int cpu, u32 offset, u8 *dst)
+static int __gic_dist_readb(struct gic_state * s, int cpu, u32 offset, u8 *dst)
 {
 	u32 done = 0, i, irq, mask;
 
@@ -428,7 +455,7 @@ static int gic_dist_readb(struct gic_state * s, int cpu, u32 offset, u8 *dst)
 	return VMM_OK;
 }
 
-static int gic_dist_writeb(struct gic_state * s, int cpu, u32 offset, u8 src)
+static int __gic_dist_writeb(struct gic_state *s, int cpu, u32 offset, u8 src)
 {
 	u32 done = 0, i, irq, mask, cm;
 
@@ -593,10 +620,6 @@ static int gic_dist_writeb(struct gic_state * s, int cpu, u32 offset, u8 src)
 		};
 	}
 
-	if (done) {
-		gic_update(s);
-	}
-
 	if (!done) {
 		return VMM_EFAIL;
 	}
@@ -607,36 +630,39 @@ static int gic_dist_writeb(struct gic_state * s, int cpu, u32 offset, u8 src)
 static int gic_dist_read(struct gic_state *s, int cpu, u32 offset, u32 *dst)
 {
 	int rc = VMM_OK, i;
+	irq_flags_t flags;
 	u8 val;
 
 	if (!s || !dst) {
 		return VMM_EFAIL;
 	}
 
-	vmm_spin_lock(&s->lock);
+	vmm_spin_lock_irqsave(&s->lock, flags);
+
 	*dst = 0;
 	for (i = 0; i < 4; i++) {
-		if ((rc = gic_dist_readb(s, cpu, 
-					 offset + i, &val))) {
+		if ((rc = __gic_dist_readb(s, cpu, offset + i, &val))) {
 				break;
 		}
 		*dst |= val << (i * 8);
 	}
-	vmm_spin_unlock(&s->lock);
+
+	vmm_spin_unlock_irqrestore(&s->lock, flags);
 
 	return VMM_OK;
 }
 
 static int gic_dist_write(struct gic_state *s, int cpu, u32 offset, 
-		   u32 src_mask, u32 src)
+			  u32 src_mask, u32 src)
 {
 	int rc = VMM_OK, irq, mask, i;
+	irq_flags_t flags;
 
 	if (!s) {
 		return VMM_EFAIL;
 	}
 
-	vmm_spin_lock(&s->lock);
+	vmm_spin_lock_irqsave(&s->lock, flags);
 
 	if (offset == 0xF00) {
 		/* Software Interrupt */
@@ -656,12 +682,11 @@ static int gic_dist_write(struct gic_state *s, int cpu, u32 offset,
 			break;
 		};
 		GIC_SET_PENDING(s, irq, mask);
-		gic_update(s);
 	} else {
 		src_mask = ~src_mask;
 		for (i = 0; i < 4; i++) {
 			if (src_mask & 0xFF) {
-				if ((rc = gic_dist_writeb(s, cpu, 
+				if ((rc = __gic_dist_writeb(s, cpu, 
 						offset + i, src & 0xFF))) {
 					break;
 				}
@@ -671,12 +696,14 @@ static int gic_dist_write(struct gic_state *s, int cpu, u32 offset,
 		}
 	}
 
-	vmm_spin_unlock(&s->lock);
+	__gic_update(s);
+
+	vmm_spin_unlock_irqrestore(&s->lock, flags);
 
 	return rc;
 }
 
-static int gic_cpu_read(struct gic_state * s, u32 cpu, u32 offset, u32 *dst)
+static int gic_cpu_read(struct gic_state *s, u32 cpu, u32 offset, u32 *dst)
 {
 	int rc = VMM_OK;
 
@@ -687,8 +714,6 @@ static int gic_cpu_read(struct gic_state * s, u32 cpu, u32 offset, u32 *dst)
 		return VMM_EFAIL;
 	}
 
-	vmm_spin_lock(&s->lock);
-
 	switch (offset) {
 	case 0x00: /* Control */
 		*dst = s->cpu_enabled[cpu];
@@ -713,12 +738,10 @@ static int gic_cpu_read(struct gic_state * s, u32 cpu, u32 offset, u32 *dst)
 		break;
 	}
 
-	vmm_spin_unlock(&s->lock);
-
 	return rc;
 }
 
-static int gic_cpu_write(struct gic_state * s, u32 cpu, u32 offset, 
+static int gic_cpu_write(struct gic_state *s, u32 cpu, u32 offset, 
 		  u32 src_mask, u32 src)
 {
 	int rc = VMM_OK;
@@ -732,14 +755,14 @@ static int gic_cpu_write(struct gic_state * s, u32 cpu, u32 offset,
 
 	src = src & ~src_mask;
 
-	vmm_spin_lock(&s->lock);
-
 	switch (offset) {
 	case 0x00: /* Control */
 		s->cpu_enabled[cpu] = src & 0x1;
+		gic_update(s);
 		break;
 	case 0x04: /* Priority mask */
 		s->priority_mask[cpu] = src & 0xFF;
+		gic_update(s);
 		break;
 	case 0x08: /* Binary Point */
 		/* ??? Not implemented.  */
@@ -752,10 +775,6 @@ static int gic_cpu_write(struct gic_state * s, u32 cpu, u32 offset,
 		break;
 	};
 
-	gic_update(s);
-
-	vmm_spin_unlock(&s->lock);
-
 	return rc;
 }
 
@@ -819,7 +838,7 @@ static int gic_emulator_read(struct vmm_emudev *edev,
 {
 	int rc = VMM_OK;
 	u32 regval = 0x0;
-	struct gic_state * s = edev->priv;
+	struct gic_state *s = edev->priv;
 
 	rc = gic_reg_read(s, offset, &regval);
 
@@ -850,7 +869,7 @@ static int gic_emulator_write(struct vmm_emudev *edev,
 {
 	int i;
 	u32 regmask = 0x0, regval = 0x0;
-	struct gic_state * s = edev->priv;
+	struct gic_state *s = edev->priv;
 
 	switch (src_len) {
 	case 1:
@@ -881,8 +900,9 @@ static int gic_emulator_write(struct vmm_emudev *edev,
 int gic_state_reset(struct gic_state *s)
 {
 	u32 i;
+	irq_flags_t flags;
 
-	vmm_spin_lock(&s->lock);
+	vmm_spin_lock_irqsave(&s->lock, flags);
 
 	/*
 	 * We should not reset level as for host to guest IRQ might
@@ -897,7 +917,11 @@ int gic_state_reset(struct gic_state *s)
 	}
 
 	for (i = 0; i < GIC_NUM_CPU(s); i++) {
-		s->priority_mask[i] = 0xf0;
+		if (s->type == GIC_TYPE_11MPCORE) {
+			s->priority_mask[i] = 0xf0;
+		} else {
+			s->priority_mask[i] = 0x0;
+		}
 		s->current_pending[i] = 1023;
 		s->running_irq[i] = 1023;
 		s->running_priority[i] = 0x100;
@@ -910,7 +934,7 @@ int gic_state_reset(struct gic_state *s)
 	}
 	s->enabled = 0;
 
-	vmm_spin_unlock(&s->lock);
+	vmm_spin_unlock_irqrestore(&s->lock, flags);
 
 	return VMM_OK;
 }
@@ -940,6 +964,22 @@ static u32 gic_configs[][14] = {
 		/* dist_offset */ 0x1000,
 		/* dist_length */ 0x1000,
 	},
+	{ 
+		/* num_irq */ 96,
+		/* num_base_irq */ 0,
+		/* id0 */ 0x90, 
+		/* id1 */ 0x13, 
+		/* id2 */ 0x04, 
+		/* id3 */ 0x00, 
+		/* id4 */ 0x0d, 
+		/* id5 */ 0xf0, 
+		/* id6 */ 0x05, 
+		/* id7 */ 0xb1,
+		/* cpu_offset */ 0x0,
+		/* cpu_length */ 0x100,
+		/* dist_offset */ 0x1000,
+		/* dist_length */ 0x1000,
+	},
 	{ 
 		/* num_irq */ 96,
 		/* num_base_irq */ 0,
@@ -956,9 +996,25 @@ static u32 gic_configs[][14] = {
 		/* dist_offset */ 0x1000,
 		/* dist_length */ 0x1000,
 	},
+	{ 
+		/* num_irq */ 128,
+		/* num_base_irq */ 0,
+		/* id0 */ 0x90, 
+		/* id1 */ 0x13, 
+		/* id2 */ 0x04, 
+		/* id3 */ 0x00, 
+		/* id4 */ 0x0d, 
+		/* id5 */ 0xf0, 
+		/* id6 */ 0x05, 
+		/* id7 */ 0xb1,
+		/* cpu_offset */ 0x2000,
+		/* cpu_length */ 0x1000,
+		/* dist_offset */ 0x1000,
+		/* dist_length */ 0x1000,
+	},
 };
 
-struct gic_state* gic_state_alloc(struct vmm_guest *guest,
+struct gic_state *gic_state_alloc(struct vmm_guest *guest,
 			   enum gic_type type,
 			   u32 num_cpu,
 			   bool is_child_pic,
@@ -990,6 +1046,7 @@ struct gic_state* gic_state_alloc(struct vmm_guest *guest,
 	s->num_cpu = num_cpu;
 	s->num_irq = gic_configs[type][0];
 	s->num_base_irq = gic_configs[type][1];
+	s->type = type;
 	s->id[0] = gic_configs[type][2];
 	s->id[1] = gic_configs[type][3];
 	s->id[2] = gic_configs[type][4];
@@ -1048,7 +1105,7 @@ static int gic_emulator_probe(struct vmm_guest *guest,
 {
 	u32 attrlen;
 	const char *attr;
-	struct gic_state * s;
+	struct gic_state *s;
 	bool is_child_pic;
 	enum gic_type type;
 	u32 *parent_irq;
@@ -1062,11 +1119,14 @@ static int gic_emulator_probe(struct vmm_guest *guest,
 	}
 
 	attr = vmm_devtree_attrval(edev->node, "parent_irq");
-	attrlen = vmm_devtree_attrlen(edev->node, "parent_irq");
-	num_cpu = (attrlen / sizeof(u32));
 	if (!attr) {
 		return VMM_EFAIL;
 	}
+	attrlen = vmm_devtree_attrlen(edev->node, "parent_irq");
+	num_cpu = (attrlen / sizeof(u32));
+	if (guest->vcpu_count < num_cpu) {
+		num_cpu = guest->vcpu_count;
+	}
 	parent_irq = (u32 *)attr;
 	
 	type = (enum gic_type)(eid->data);
@@ -1080,7 +1140,7 @@ static int gic_emulator_probe(struct vmm_guest *guest,
 
 static int gic_emulator_remove(struct vmm_emudev *edev)
 {
-	struct gic_state * s = edev->priv;
+	struct gic_state *s = edev->priv;
 
 	gic_state_free(s);
 	edev->priv = NULL;
@@ -1097,6 +1157,10 @@ static struct vmm_emuid gic_emuid_table[] = {
 	  .compatible = "vexpress,gic", 
 	  .data = (void *)GIC_TYPE_VEXPRESS,
 	},
+	{ .type = "pic", 
+	  .compatible = "vexpress,gicv2", 
+	  .data = (void *)GIC_TYPE_VEXPRESS_V2,
+	},
 	{ /* end of list */ },
 };
 
